//
// zedit - simple text editor
// Copyright (C) 2004-2005 Bryan Beicker <tokiko@tokiko.net>
//
// This file is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License (Version 2)
// as published by the Free Software Foundation.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include <gtk/gtk.h>		// gtk
#include <sys/types.h>		// open()
#include <sys/stat.h>		// open()
#include <fcntl.h>		// open()
#include <stdio.h>		// BUFSIZE
#include <unistd.h>		// write()
#include "main.h"
#include "zedit.xpm"


//
// write a string to a file
//
int slap(char * path, char * data, int len)
{
	int fd;
	int bytes;
	int size;
	int count = 0;
	
	fd = open(path, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
	if (fd == -1) {
		return FALSE;
	}
	
	while (count < len) {
		size = len - count;
		if (size > BUFSIZ) {
			size = BUFSIZ;
		}
		bytes = write(fd, data + count, size);
		if (bytes == -1) {
			g_print("error writing file\n");
			close(fd);
			return FALSE;
		}
		count += bytes;
	}
	
	close(fd);
	return TRUE;
}


void main_load()
{
	file_name = NULL;
	dirty = FALSE;
}


//
// ask to save the file if dirty
//
//   return TRUE if the buffer is still dirty (they want to cancel)
//   return FALSE if they are finished with the buffer
//
gboolean dirty_check()
{
	if (dirty) {
		switch (save_question_dialog(zedit_window)) {
			case ZGTK_DIALOG_DONT_SAVE:
				return FALSE;
			case ZGTK_DIALOG_CANCEL:
				return TRUE;
			case ZGTK_DIALOG_SAVE:
				save_button_click();
				if (dirty) {
					return TRUE;
				}
				return FALSE;
		}
	}
	
	return FALSE;
}


void open_file(char * path)
{
	GError * e;
	gsize len;
	gchar * body;

	if (!g_file_get_contents(path, &body, &len, &e)) {
		g_print("error opening file");
		return;
	}
	
	if (!g_utf8_validate(body, -1, NULL)) {
		error_dialog(zedit_window, "invalid UTF-8");
		return;
	}
	
	g_free(file_name);
	file_name = path;
	
	gtk_text_buffer_set_text(buffer, body, len);
	g_free(body);
	
	gtk_window_set_title(GTK_WINDOW(zedit_window), file_name);
	dirty = FALSE;
}


//
// window manager is trying to close the window
//
gboolean zedit_unload(GtkWidget * window, GdkEvent * event, gpointer data)
{
	if (dirty_check()) {
		// prevent the window from closing
		return TRUE;
	}
	
	// let the window close
	return FALSE;
}


//
// exit the program when the window dies
//
void zedit_destroy(GtkWidget * window, gpointer data)
{
	gtk_main_quit();
}


void new_button_click()
{
	if (dirty_check()) {
		return;
	}
	
	gtk_text_buffer_set_text(buffer, "", -1);
	g_free(file_name);
	file_name = NULL;
	dirty = FALSE;
	gtk_window_set_title(GTK_WINDOW(zedit_window), "zedit");
}


void open_button_click()
{
	GtkWidget * dialog;
	gchar * path;
	
	if (dirty_check()) {
		return;
	}

	dialog = gtk_file_chooser_dialog_new("open", GTK_WINDOW(zedit_window), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		gtk_widget_destroy(dialog);
		open_file(path);
	} else {
		gtk_widget_destroy(dialog);
	}
}


void save_button_click()
{
	GtkTextIter start, end;
	gchar * body;
	int len;

	if (file_name == NULL) {
		return save_as_button_click(NULL, NULL, NULL);
	}

	gtk_text_buffer_get_iter_at_offset(buffer, &start, 0);
	len = gtk_text_buffer_get_char_count(buffer);
	gtk_text_buffer_get_iter_at_offset(buffer, &end, len);
	body = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
	
	slap(file_name, body, len);
	gtk_window_set_title(GTK_WINDOW(zedit_window), file_name);
	g_free(body);
	dirty = FALSE;
}


void save_as_button_click()
{
	GtkWidget * dialog;
	
	dialog = gtk_file_chooser_dialog_new("save", GTK_WINDOW(zedit_window), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		g_free(file_name);
		file_name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		save_button_click(NULL, NULL, NULL);
	}
	gtk_widget_destroy(dialog);
}


void cut_button_click()
{
	gtk_text_buffer_cut_clipboard(buffer, gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", TRUE)), TRUE);
}


void copy_button_click()
{
	gtk_text_buffer_copy_clipboard(buffer, gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", TRUE)));
}


void paste_button_click()
{
	gtk_text_buffer_paste_clipboard(buffer, gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", TRUE)), NULL, TRUE);
}


void about_button_click()
{
	about_dialog(zedit_window, zedit_xpm, "zedit " VERSION, "simple text editor", "Copyright (C) 2004-2005 Bryan Beicker");
}


void textbox_change()
{
	gchar * title;
	
	if (dirty == FALSE) {
		if (file_name != NULL) {
			title = g_strdup_printf("%s*", file_name);
			gtk_window_set_title(GTK_WINDOW(zedit_window), title);
			g_free(title);
		}
		dirty = TRUE;
	}
}


void textbox_selection_change()
{
	guint col;
	guint row;
	GtkTextIter iter;
	GtkTextIter start;
	gchar * pos;
	
	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
	row = gtk_text_iter_get_line(&iter);
	
	col = 0;
	start = iter;
        gtk_text_iter_set_line_offset(&start, 0);
	
	while (!gtk_text_iter_equal(&start, &iter)) {
		if (gtk_text_iter_get_char(&start) == '\t') {
			col += (8 - (col  % 8));
		} else {
			col++;
		}
		gtk_text_iter_forward_char(&start);
	}

	pos = g_strdup_printf(" line: %d  col: %d", row + 1, col + 1);
	gtk_statusbar_pop(GTK_STATUSBAR(status), 0);
	gtk_statusbar_push(GTK_STATUSBAR(status), 0, pos);
	g_free(pos);
}
